searchData={"content_type":"text/markdown","items":[{"doc":"Design By Contract for Elixir.\n\nBond provides thorough support for contract programming (also known as\n\"Design By Contract\") for Elixir.\n\nAs described on [Wikipedia](https://en.wikipedia.org/wiki/Design_by_contract):\n\n> Design by contract (DbC), also known as contract programming, programming by\n> contract and design-by-contract programming, is an approach for designing\n> software.\n>\n> It prescribes that software designers should define formal, precise and\n> verifiable interface specifications for software components, which extend the\n> ordinary definition of\n> [abstract data types](https://en.wikipedia.org/wiki/Abstract_data_type) with\n> preconditions, postconditions and invariants. These specifications are referred\n> to as \"contracts\", in accordance with a conceptual metaphor with the conditions\n> and obligations of business contracts.\n>\n> The term was coined by\n> [Bertrand Meyer](https://en.wikipedia.org/wiki/Bertrand_Meyer) in connection\n> with his design of the\n> [Eiffel programming language](https://en.wikipedia.org/wiki/Eiffel_(programming_language))\n> and first described in various articles starting in 1986 and the two successive\n> editions (1988, 1997) of his book\n> [_Object-Oriented Software Construction_](https://en.wikipedia.org/wiki/Object-Oriented_Software_Construction).\n>\n> Design by contract has its roots in work on\n> [formal verification](https://en.wikipedia.org/wiki/Formal_verification),\n> [formal specification](https://en.wikipedia.org/wiki/Formal_specification) and\n> [Hoare logic](https://en.wikipedia.org/wiki/Hoare_logic).\n\nBond applies the central ideas of contract programming to Elixir and provides\nsupport for attaching preconditions and postconditions to function definitions\nand conditionally evaluating them based on compile-time configuration.","ref":"Bond.html","title":"Bond","type":"module"},{"doc":"Bond introduces two special annotations that can be used to define\ncontracts for functions:\n\n- `@pre` for defining function _preconditions_\n- `@post` for defining function _postconditions_\n\nBoth of these annotations allow for attaching one or more assertions (with\noptional labels) to functions. These assertions are attached to functions at\ncompile-time and evaluated at run-time.\n\nTo use these annotations and other features of Bond, you must `use` the `Bond`\nmodule in your own module(s). For example:\n\n```elixir\ndefmodule Math do\n  use Bond\n\n  @pre numeric_x: is_number(x), non_negative_x: x >= 0\n  @post float_result: is_float(result),\n        non_negative_result: result >= 0.0,\n        \"sqrt of 0 is 0\": (x == 0) ~> (result === 0.0),\n        \"sqrt of 1 is 1\": (x == 1) ~> (result === 1.0),\n        \"x > 1 implies result smaller than x\": (x > 1) ~> (result   #### `use Bond` {: .info}\n>\n> When you `use Bond`, the `Bond` module will override several `Kernel` macros\n> in order to support attaching preconditions and postconditions to functions.\n> Specifically:\n>\n> - `Kernel.@/1` is overridden by `Bond.@/1`\n> - `Kernel.def/2` is overridden by `Bond.def/2`\n> - `Kernel.defp/2` is overridden by `Bond.defp/2`\n>\n> `use Bond` will also import the `Bond` module so that the `check/1` and\n> `check/2` macros are available for use.\n>\n> Additionally, the `Bond.Predicates` module is automatically imported for all\n> preconditions, postconditions, and checks, so that the predicate functions and\n> operators that are defined therein can be used for assertions.\n> `Bond.Predicates` can be explicitly imported into modules for use outside of\n> assertions.","ref":"Bond.html#module-usage","title":"Usage - Bond","type":"module"},{"doc":"Assertions in Bond are conditional Elixir expressions, optionally associated\nwith a textual label (either an atom or a string). These assertions may appear\nin `@pre` or `@post` expressions, or in calls to `check/1` or `check/2`.\n\nBond offers considerable flexibility in its assertion syntax; assertions may\ntake any of the following forms:\n\n- `expression` - a \"bare\" expression without any associated label\n- `label, expression` - an expression preceded by a string or atom label\n- `expression, label` - an expression followed by a string or atom label\n- `label_1: expression_1, label_2: expression_2` - a keyword list with labels as\n  the keys and expressions as the associated values\n\nBond also provides the `Bond.Predicates` module with predicates that are often\nuseful in assertions. These include an \"exclusive or\" predicate and a logical\nimplication predicate. The `Bond.Predicates` module is automatically imported\nfor preconditions, postconditions, and `check` assertions. See the\n`Bond.Predicates` documentation for further details.\n\n## `old` expressions\n\n`old` expressions allow postconditions to access the value of any arbitrary\nexpression _prior to_ execution of the function body. Postconditions are\n\"pre-compiled\" in such a way that any `old` expressions that appear in\nassertions are resolved to the value that they had at the start of function\nexecution.\n\nWhile this facility is not particularly relevant for purely functional code,\nit can be useful for stateful components of an application.\n\nFor example, imagine a simple, stateful `Counter` module that uses an `Agent`\nto store the current count (some Agent code omitted for brevity):\n\n```elixir\ndefmodule Counter do\n  use Bond\n\n  def get_count(agent) do\n    Agent.get(agent, & &1)\n  end\n\n  @post count_incremented_by_1: get_count(agent) == old(get_count(agent)) + 1\n  def increment_count(agent) do\n    Agent.update(agent, &(&1 + 1))\n  end\nend\n```\n\nNotice how the `old` expression captures the value of `get_count/1` prior to\nexecution of the function, and this value is used to verify that the value of\n`get_count/1` has been updated as expected.\n\nNote, however, that there is a potential race condition in the above code.\nSince Agents are inherently concurrent, it is possible that another call to\n`increment_count/1` is interleaved between execution of the function body and\nthe call to `get_count/1` that appears in the postcondition. In this scenario\nthe postcondition would fail because the new value of `get_count/1` would be\nat least 2 greater than the old value captured in the postcondition, rather\nthan exactly 1 greater as specified in the `count_incremented_by_1` assertion.\n\nAs a first attempt to alleviate this race condition we can update the\n`increment_count/1` function so that it returns the updated count as its result\nand use that result in the postcondition directly:\n\n```elixir\n  @post returns_updated_count: result == old(get_count(agent)) + 1\n  def increment_count(agent) do\n    Agent.get_and_update(agent, fn count ->\n      new_count = count + 1\n      {new_count, new_count}\n    end)\n  end\n```\n\nIn this version we utilize `Agent.get_and_update/3` to update the counter and\nreturn the updated counter value in one operation. The new counter value is the\n`result` of the function which can be used in postconditions. The\n`returns_updated_count` assertion compares this `result` to the `old` value of\n`get_count/1` to ensure that it was incremented by exactly 1.\n\nHowever, as you may have noticed, it is still possible for another call to\n`increment_count/1` to be interleaved between the call to `get_count/1` in the\n`old` expression of the postcondition and the call to `Agent.get_and_update/3`\nin the function body. Alas, there is no way to \"lock\" an Agent over multiple\noperations to ensure that there are no concurrent updates to the Agent state.\nTherefore, our only choice is to soften the guarantee made by our\npostcondition:\n\n```elixir\n  @post count_increased: get_count(agent) > old(get_count(agent))\n  def increment_count(agent) do\n    Agent.update(agent, &(&1 + 1))\n  end\n```\n\nThe `count_increased` assertion in the postcondition now guarantees only that\nthe new value of `get_count/1` is strictly greater than the old value. This\nassertion always holds true regardless of the number of concurrent state\nupdates to the counter.\n\nAlthough this assertion is not as strong as the `count_incremented_by_1`\nassertion in the original version, it is the strongest we can provide given\nthe possibility of concurrent state updates.\n\nFuture versions of Bond may provide stronger support for stateful contracts\nin the form of _invariants_ for structs and/or stateful processes, although\nthis is still a subject of research.","ref":"Bond.html#module-assertion-syntax","title":"Assertion syntax - Bond","type":"module"},{"doc":"Contracts in the form of preconditions and postconditions are part of the\npublic interface for a module in the same way that function signatures and\ntypespecs are. Therefore, it is essential that contracts are included as part\nof the documentation for modules and functions.\n\nBond automatically appends `Preconditions` and `Postconditions` section to the\ndocumentation for any function that defines any preconditions or postconditions\nand has a `@doc` attribute. These two generated sections include all of the\nassertions specified in the function contract as nicely formatted Elixir code.\nFurthermore, contract documentation is generated even if run-time assertion\nchecking is disabled via configuration. Therefore, it is not necessary to\nexplicitly document preconditions or postconditions in the `@doc` for a\nfunction unless greater detail is warranted.\n\nThe contract documentation is visible not only in documentation generated by\n`ex_doc` but also in code editing environments that are able to display\nfunction docs directly in the editor, such as with the `K` command in Vim or\non mouse hover in VS Code.","ref":"Bond.html#module-documenting-contracts","title":"Documenting contracts - Bond","type":"module"},{"doc":"Override `Kernel.@/1` to support `@pre` and `@post` annotations.\n\nSee the `Bond` module docs for the syntax of `@pre` and `@post` annotations.","ref":"Bond.html#@/1","title":"Bond.@/1","type":"macro"},{"doc":"Check an assertion or a keyword list of assertions for validity.\n\nReturns the result(s) of the assertion(s) if satisfied, or raises a `Bond.CheckError` exception\nif any assertions are not satisfied.","ref":"Bond.html#check/1","title":"Bond.check/1","type":"macro"},{"doc":"iex> check 1 == 1.0\n    true\n    iex> check 1 == 1.0, \"integer 1 is equal to float 1.0\"\n    true\n    iex> check \"integer 1 is equal to float 1.0\", 1 == 1.0\n    true\n    iex> check tautology: 1 == 1\n    [true]\n    iex> check \"1 is 1\": 1 == 1, \"2 is 2\": 2 == 2\n    [true, true]","ref":"Bond.html#check/1-examples","title":"Examples - Bond.check/1","type":"macro"},{"doc":"Check a single labelled assertion for validity.\n\nSee `check/1` for details and examples.","ref":"Bond.html#check/2","title":"Bond.check/2","type":"macro"},{"doc":"Override `Kernel.def/2` to support wrapping with preconditions and postconditions.","ref":"Bond.html#def/2","title":"Bond.def/2","type":"macro"},{"doc":"Override `Kernel.defp/2` to support wrapping with preconditions and postconditions.","ref":"Bond.html#defp/2","title":"Bond.defp/2","type":"macro"},{"doc":"Type to represent a compile-time quoted assertion expression, which must be a valid Elixir\nexpression that, when unquoted, evaluates to a `t:boolean/0` or `t:as_boolean/1` value.","ref":"Bond.html#t:assertion_expression/0","title":"Bond.assertion_expression/0","type":"type"},{"doc":"Type to represent a label for an assertion, which must be a compile-time atom or string.","ref":"Bond.html#t:assertion_label/0","title":"Bond.assertion_label/0","type":"type"},{"doc":"Subset of `Macro.Env` struct that excludes fields that, according to the documentation, \"are\nprivate to Elixir's macro expansion mechanism\".","ref":"Bond.html#t:env/0","title":"Bond.env/0","type":"type"},{"doc":"Predicate functions and operators that are useful in assertions and contract specifications.\n\nThis module is automatically imported for all \nTo use the infix operator versions of the predicates, this module must be imported in the\nusing module.","ref":"Bond.Predicates.html","title":"Bond.Predicates","type":"module"},{"doc":"Pattern matching operator: equivalent to `match?(pattern, expression)`.","ref":"Bond.Predicates.html#%3C~/2","title":"Bond.Predicates.<~/2","type":"macro"},{"doc":"iex> {:ok, %Date{}} <~ Date.new(1974, 6, 6) \n    true\n    iex> {:error, _} <~ Date.new(-1, -1, -1)\n    true","ref":"Bond.Predicates.html#%3C~/2-examples","title":"Examples - Bond.Predicates.<~/2","type":"macro"},{"doc":"Logical implication: does `p` imply `q`?\n\nFor an infix operator version of logical implication see `~>/2`.","ref":"Bond.Predicates.html#implies?/2","title":"Bond.Predicates.implies?/2","type":"function"},{"doc":"iex> implies?(true, true)\n    true\n    iex> implies?(true, false)\n    false\n    iex> implies?(false, true)\n    true\n    iex> implies?(false, false)\n    true","ref":"Bond.Predicates.html#implies?/2-examples","title":"Examples - Bond.Predicates.implies?/2","type":"function"},{"doc":"Logical exclusive or: is either `p` or `q` true, but not both?\n\nFor an infix operator version of exclusive or see `|||/2`.","ref":"Bond.Predicates.html#xor/2","title":"Bond.Predicates.xor/2","type":"function"},{"doc":"iex> xor(true, true)\n    false\n    iex> xor(true, false)\n    true\n    iex> xor(false, true)\n    true\n    iex> xor(false, false)\n    false","ref":"Bond.Predicates.html#xor/2-examples","title":"Examples - Bond.Predicates.xor/2","type":"function"},{"doc":"Logical exclusive or operator: `p ||| q` means `xor(p, q)`.\n\nNote that the `|||` operator has higher precedence than many other operators and it may be\nnecessary to parenthesize the expressions on either side of the operator to get the\nexpected result.","ref":"Bond.Predicates.html#%7C%7C%7C/2","title":"Bond.Predicates.|||/2","type":"function"},{"doc":"iex> true ||| true\n    false\n    iex> true ||| false\n    true\n    iex> false ||| true\n    true\n    iex> false ||| false\n    false\n    iex> x = 2\n    2\n    iex> y = 4\n    4\n    iex> (x - y < 0) ||| (y <= x)\n    true","ref":"Bond.Predicates.html#%7C%7C%7C/2-examples","title":"Examples - Bond.Predicates.|||/2","type":"function"},{"doc":"Logical implication operator: `p ~> q` means `implies?(p, q)`.\n\nNote that the `~>` operator has higher precedence than many other operators and it may be\nnecessary to parenthesize the expressions on either side of the operator to get the\nexpected result.","ref":"Bond.Predicates.html#~%3E/2","title":"Bond.Predicates.~>/2","type":"function"},{"doc":"iex> true ~> true\n    true\n    iex> true ~> false\n    false\n    iex> false ~> true\n    true\n    iex> false ~> false\n    true\n    iex> x = 2\n    2\n    iex> y = 4\n    4\n    iex> (x - y < 0) ~> (y > x)\n    true","ref":"Bond.Predicates.html#~%3E/2-examples","title":"Examples - Bond.Predicates.~>/2","type":"function"},{"doc":"Exception raised when a `Bond.check/2` assertion fails.","ref":"Bond.CheckError.html","title":"Bond.CheckError","type":"exception"},{"doc":"The `Bond.CheckError` exception type.","ref":"Bond.CheckError.html#t:t/0","title":"Bond.CheckError.t/0","type":"type"},{"doc":"Exception raised when a function postcondition fails.","ref":"Bond.PostconditionError.html","title":"Bond.PostconditionError","type":"exception"},{"doc":"The `Bond.PostconditionError` exception type.","ref":"Bond.PostconditionError.html#t:t/0","title":"Bond.PostconditionError.t/0","type":"type"},{"doc":"Exception raised when a function precondition fails.","ref":"Bond.PreconditionError.html","title":"Bond.PreconditionError","type":"exception"},{"doc":"The `Bond.PreconditionError` exception type.","ref":"Bond.PreconditionError.html#t:t/0","title":"Bond.PreconditionError.t/0","type":"type"},{"doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright [yyyy] [name of copyright owner]\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html","title":"License","type":"extras"},{"doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](http://keepachangelog.com/en/1.0.0/)\nand this project adheres to [Semantic Versioning](http://semver.org/spec/v2.0.0.html).","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"# Getting Started\n\nThis guide provides installation and basic usage information for getting started\nquickly with Bond.\n\nRefer to the `Bond` module docs for detailed usage instructions and examples.","ref":"getting-started.html","title":"Getting Started","type":"extras"},{"doc":"`bond` can be installed by adding it to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:bond, \"~> 0.8.1\"}\n  ]\nend\n```","ref":"getting-started.html#installation","title":"Installation - Getting Started","type":"extras"},{"doc":"Use Bond to define preconditions and postconditions (collectively referred to as\n\"contracts\") to functions.\n\n```elixir\ndefmodule Math do\n  use Bond\n\n  @pre numeric_x: is_number(x), non_negative_x: x >= 0\n  @post float_result: is_float(result),\n        non_negative_result: result >= 0.0,\n        \"sqrt of 0 is 0\": (x == 0) ~> (result === 0.0),\n        \"sqrt of 1 is 1\": (x == 1) ~> (result === 1.0),\n        \"x > 1 implies result smaller than x\": (x > 1) ~> (result < x)\n  def sqrt(x), do: :math.sqrt(x)\nend\n```","ref":"getting-started.html#quick-start","title":"Quick start - Getting Started","type":"extras"},{"doc":"# About\n\nBond provides support for contract programming (also known as\n\"Design By Contract\") for Elixir.\n\nThe primary goal for Bond is to provide the most feature-complete and thoroughly\ndocumented Design By Contract library for Elixir, with a concise and flexible\nsyntax for specifying contracts.\n\nCurrent and planned features include:\n\n- [x] Function preconditions with `@pre`\n- [x] Function postconditions with `@post`\n- [x] \"old\" expressions in postconditions\n- [x] \"check\" expressions for arbitrary assertions within a function body\n- [x] Predicates (such as `implies?` and `xor`) for use in assertions\n- [x] Detailed assertion failure reporting\n- [x] Incorporation of preconditions and postconditions into @doc for function\n- [ ] Conditional compilation of contracts per environment\n- [ ] More detailed assertion failure reporting, including color coding à la `ExUnit`\n- [ ] Invariants for structs and/or stateful processes (if possible)","ref":"about.html","title":"About","type":"extras"},{"doc":"# History\n\nThe ideas and philosophy of Design By Contract are, of course, not original to\nBond, having first been developed by Bertrand Meyer in the mid 1980s. Meyer's\nwork was itself based on earlier work in the field of formal specification and\nverification, by Tony Hoare and others.\n\nNor is Bond the first or only attempt at bringing the ideas of contract\nprogramming to Elixir.","ref":"history.html","title":"History","type":"extras"},{"doc":"In 2016, Elba Sanchez Marquez presented at various conferences and in blog posts\nthe [elixir-contracts](https://github.com/epsanchezma/elixir-contracts) library,\nwhich she collaboratively developed with Guillermo Iguaran. These conference\npresentations and the `elixir-contracts` library represented a good introduction\nto contract programming for the Elixir community. However, development of the\nlibrary ceased after a mere 10 commits to its GitHub repository.\n\nIn 2017, Dariusz Gawdzik created his\n[ExContract](https://github.com/JDUnity/ex_contract) library, with contributions\nfrom myself and others. This library took ideas from the `elixir-contracts`\nlibrary as a starting point and produced a more complete implementation and\nthorough documentation. However, development of `ExContract`, too, halted in\n2019 after 19 commits, and with\n[one unmerged pull request](https://github.com/JDUnity/ex_contract/pull/4)\nfrom [yours truly](https://github.com/jvoegele).\n\nAt ElixirConf 2019, Chris Keathley gave a talk titled\n[Contracts for Building Reliable Systems](https://www.youtube.com/watch?v=tpo3JUyVIjQ),\nwhich presented the ideas of Design By Contract using `ExContract` for\nits examples, as well as the distinct but related idea of data specification\nusing his library [Norm](https://github.com/elixir-toniq/norm), which we'll come\nback to shortly. (I happened to be in the audience for this talk and was pleased\nto see that a library that I was actively contributing to at the time was being\npresented to the Elixir community at large.) Shortly thereafter, Keathley\nreleased his own library for Design By Contract in Elixir:\n[Oath](https://github.com/keathley/oath). Oath supports the same basic notions\nas earlier DbC libraries of having preconditions and postconditions attached to\nfunctions, but does so using a different (and rather verbose) syntax based on\nthe popular [decorator](https://github.com/arjan/decorator) library. Oath was\nthe first Elixir contracts library that allowed for conditionally enabling\ncontracts on a per-environment basis, so that contracts could be compiled into\nthe code in dev and test environments, but disabled in production so that\nthere are no runtime costs associated with contracts. At the time of writing,\nOath is currently the most popular contracts library for Elixir, but like the\nothers before it, it has been largely unmaintained and development has ceased\nas of April of 2021 (22 commits in this time).","ref":"history.html#prior-art-and-related-work","title":"Prior art and related work - History","type":"extras"},{"doc":"After growing dissatisfied with the existing offerings for Design By Contract in\nElixir, I decided to create my own.\n\n`ExContract` provided the most complete implementation and documentation, but\nhaving tried and failed to revive development I gave up on this library.\n\nOath, even though it is not actively maintained, is reasonably complete and its\nsupport for conditional compilation of contracts is an attractive feature.\nHowever, its verbose syntax and dearth of documentation are both barriers to its\nwidespread adoption.\n\nEnter [Bond](https://github.com/jvoegele/bond).","ref":"history.html#raison-d-%C3%AAtre","title":"Raison d'être - History","type":"extras"},{"doc":"The initial implementation was based partially on `ExContract`, or more\nprecisely, on my\n[unmerged pull request](https://github.com/JDUnity/ex_contract/pull/4) to it.\nThe implementation of preconditions and postconditions as `@pre` and `@post`\nmodule attributes is accomplished by selectively overriding Elixir's\n`Kernel.@/1` macro, a technique discovered by studying the source code to\n[Norm's @contract construct](https://github.com/elixir-toniq/norm/blob/be1c31bc33ae10723b3d4fe8b9b3a2ffce90b710/lib/norm/contract.ex#L57-L65).","ref":"history.html#credits","title":"Credits - History","type":"extras"},{"doc":"A closely related notion to contract programming is \"data specification\", an\nidea that has become popular in the functional programming community via\n[clojure.spec](https://clojure.org/about/spec). In Elixir, the de facto standard\nfor data specification and validation is the aforementioned\n[Norm](https://github.com/elixir-toniq/norm) library from Chris Keathley, et al.\nAlthough Bond does not currently support data specification, if there is ever\na demonstrated need for integrating Design By Contract with data specification,\nBond may be extended to support it, perhaps with a `Bond.Spec` module. In the\nmeantime, it is entirely possible to use Norm in conjunction with Bond.","ref":"history.html#a-note-on-data-specification-and-validation","title":"A note on data specification and validation - History","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}